use AdventureWorks2008R2
go

--	1. Cross Apply
--	1a. Subquery in the select statement
SELECT	  soh.SalesOrderID
		, soh.OrderDate
		, (SELECT MAX(sod.UnitPrice) FROM Sales.SalesOrderDetail sod WHERE soh.SalesOrderID = sod.SalesOrderID) as MaxUnitPrice
FROM	Sales.SalesOrderHeader AS soh
where	soh.SalesOrderID = 43659

--	1b. Cross Apply the same query 
SELECT	  soh.SalesOrderID
		, soh.OrderDate
		, stat.max_unit_price
FROM	Sales.SalesOrderHeader AS soh
		CROSS APPLY
		(
			 SELECT   MAX(sod.UnitPrice) as max_unit_price
			FROM	Sales.SalesOrderDetail AS sod
			WHERE	soh.SalesOrderID = sod.SalesOrderID
		) as stat
where   soh.SalesOrderID = 43659

--	1c.	Add an additional aggregation
--	with an additional subquery
SELECT	  soh.SalesOrderID
		, soh.OrderDate
		, (SELECT MAX(sod.UnitPrice) FROM Sales.SalesOrderDetail sod WHERE soh.SalesOrderID = sod.SalesOrderID) as MaxUnitPrice
		, (SELECT SUM(LineTotal) FROM Sales.SalesOrderDetail sod WHERE soh.SalesOrderID = sod.SalesOrderID) as SumLineTotal
FROM	Sales.SalesOrderHeader AS soh
where	soh.SalesOrderID = 43659

--	1d. Use Cross Apply with the second aggregation
SELECT	  soh.SalesOrderID
		, soh.OrderDate
		, stat.max_unit_price
		, stat.sum_line_total
FROM	Sales.SalesOrderHeader AS soh
		CROSS APPLY
		(
			 SELECT   MAX(sod.UnitPrice) as max_unit_price
					, SUM(sod.LineTotal) as sum_line_total
			FROM	Sales.SalesOrderDetail AS sod
			WHERE	soh.SalesOrderID = sod.SalesOrderID
		) as stat
where   soh.SalesOrderID = 43659


--	1e. Add a third aggregation
--	We now have MAX, SUM and AVG 
SELECT	  soh.SalesOrderID
		, soh.OrderDate
		, (SELECT MAX(sod.UnitPrice) FROM Sales.SalesOrderDetail sod WHERE soh.SalesOrderID = sod.SalesOrderID) as MaxUnitPrice
		, (SELECT SUM(LineTotal) FROM Sales.SalesOrderDetail sod WHERE soh.SalesOrderID = sod.SalesOrderID) as SumLineTotal
		, (SELECT AVG(sod.UnitPrice) FROM Sales.SalesOrderDetail sod WHERE soh.SalesOrderID = sod.SalesOrderID) as AvgUnitPrice
FROM	Sales.SalesOrderHeader AS soh
where	soh.SalesOrderID = 43659

--	1f. How does Cross Apply perform as the aggregations go up?
SELECT	  soh.SalesOrderID
		, soh.OrderDate
		, stat.max_unit_price
		, stat.sum_line_total
		, stat.avg_unit_price
FROM	Sales.SalesOrderHeader AS soh
		CROSS APPLY
		(
			 SELECT   MAX(sod.UnitPrice) as max_unit_price
					, SUM(sod.LineTotal) as sum_line_total
					, AVG(sod.UnitPrice) as avg_unit_price
			FROM	Sales.SalesOrderDetail AS sod
			WHERE	soh.SalesOrderID = sod.SalesOrderID
		) as stat
where   soh.SalesOrderID = 43659

--	2a. Filtering with Cross Apply
--	Territory ID = 1 
--	Best 3 sales reps for first half of May 2014
select	top 3 salespersonID,
		SUM(top_soh.TotalDue) as TOP_SALES
from	sales.SalesOrderHeader as top_soh
where	top_soh.TerritoryID = 1
and		top_soh.SalesPersonID is not null
and		top_soh.OrderDate >= '2014-01-01'
and		top_soh.OrderDate < '2014-05-01'
group by top_soh.SalesPersonID
order by SUM(top_soh.TotalDue) desc

--	2b. Display all of the sales for my top 3 sales reps
--	Find the top 3 and cross apply back to the sales.SalesOrderHeader table
select	soh.*
from	sales.SalesOrderHeader as soh
		cross apply (
			select	top 3 top_soh.salespersonID
			from	sales.SalesOrderHeader as top_soh
			where	top_soh.TerritoryID = soh.TerritoryID
			and		top_soh.SalesPersonID is not null
			group by top_soh.SalesPersonID
			order by SUM(top_soh.TotalDue) desc
		) as TOP_SALES_REP 
where	soh.OrderDate >= '2014-01-01'
and		soh.OrderDate < '2014-05-01'
and		TOP_SALES_REP.SalesPersonID = soh.SalesPersonID
and		soh.TerritoryID = 1
order by soh.SalesPersonID

--	2c. A possible alternative is a CTE
--	Find the top 3 in the CTE
--	Join the CTE to the sales.SalesOrderHeader
;with TOP_SALES as (
	select	top 3 top_soh.salespersonID
			, SUM(top_soh.TotalDue) as SUM_SALES
	from	sales.SalesOrderHeader as top_soh
	where	top_soh.SalesPersonID is not null
	and		top_soh.TerritoryID = 1
	and		top_soh.OrderDate >= '2014-01-01'
	and		top_soh.OrderDate < '2014-05-01'
	group by top_soh.SalesPersonID
	order by SUM(top_soh.TotalDue) desc
)
select	soh.*
from	TOP_SALES as TS
		inner join sales.SalesOrderHeader as soh
			on TS.SalesPersonID = soh.SalesPersonID
where	soh.TerritoryID = 1
and		soh.OrderDate >= '2014-01-01'
and		soh.OrderDate < '2014-05-01'
			
			
--	Outer apply
--	3.  How much did rep 285 sell in the first half of 2014
SELECT	sod.OrderDate, sum(sod.TotalDue) as DailySales
FROM	Sales.SalesOrderHeader AS sod
WHERE	sod.SalesPersonID  = 285
and		sod.OrderDate >= '2014-01-01'
and		sod.OrderDate < '2014-07-01'
group by sod.OrderDate
order by sod.OrderDate asc

--	We see gap in the months

--	3a. Create a small calendar table
IF OBJECT_ID('tempdb..#Months') IS NOT NULL DROP TABLE #Months
create table #Months (
	CalDate	datetime not null
)

--	Fill the table with 12 months
;with Cal_CTE
as (
	select cast('2014-01-01' as datetime) as d1
	union all
	select DATEADD(MM,1,d1)
	from	Cal_CTE
	where	d1 < '2014-12-01'
	
)
insert into #Months(CalDate)
select	d1 
from	Cal_CTE

--	3b. Use Outer apply to calculate sales for the month or null
--	How much did Sales Rep 285 sell every month in the first half of 2014?
SELECT	  M1.CalDate
		, sod.DailySales
FROM	#Months as M1
OUTER APPLY
(
    SELECT	sum(sod.TotalDue) as DailySales
    FROM	Sales.SalesOrderHeader AS sod
    WHERE	M1.CalDate = sod.OrderDate
    and		sod.SalesPersonID  = 285
    group by sod.SalesPersonID
) as sod			
where	M1.CalDate < '2014-07-01'
order by M1.CalDate

--	3c. Find customers without sales this month 
--	Inner query reveals sales amount by customer
--	Outer query is the customer data
select	sc.*
		, CustomerSales.TotalSales
from	sales.Customer as sc
		outer apply (
			select	  soh.CustomerID
					, SUM(soh.TotalDue) as TotalSales
			from	sales.SalesOrderHeader as soh
			where	soh.OrderDate >= '2014-05-01'
			and		soh.OrderDate < '2014-06-01'
			and		soh.CustomerID = sc.CustomerID	
			group by soh.CustomerID
		) as CustomerSales	
order by 	CustomerSales.TotalSales asc
go

--	4a. Use a fuction for cross and outer apply
if exists(select * from sys.objects where name = 'fnCustomerMonthlySales')
begin
	drop function Sales.fnCustomerMonthlySales 
end
go

--	Create a function that returns user orders for the current month
--	We have to cheat with the data since AdventureWorks is out of data
CREATE FUNCTION Sales.fnCustomerMonthlySales(@CustomerID INT)
RETURNS @Result TABLE(CustomerID int, TotalSales MONEY)
AS
  BEGIN
	declare @SOM datetime
	declare @EOM datetime
	
	declare @MyDate datetime
	set @MyDate = GETDATE()

	--SELECT @SOM = CONVERT(VARCHAR(25),DATEADD(dd,-(DAY(@mydate)-1),@mydate),101) 
	--SELECT @EOM = CONVERT(VARCHAR(25),DATEADD(dd,-(DAY(DATEADD(mm,1,@mydate))),DATEADD(mm,1,@mydate)),101) 
	
	SELECT @SOM = '2014-05-01'
	SELECT @EOM = '2014-06-01'
	
    INSERT @Result(CustomerID, TotalSales)
    SELECT    soh.CustomerID
			, SUM(soh.TotalDue) as TotalSales
	from	sales.SalesOrderHeader as soh
	where	soh.OrderDate >= @SOM
	and		soh.OrderDate < @EOM
	and		soh.CustomerID = @CustomerID
	group by soh.CustomerID
    
    RETURN
  END
 
GO

--	4b. Find customers with sales this month, using the function
select	  sc.*
		, CustomerSales.TotalSales
from	sales.Customer as sc
		cross apply Sales.fnCustomerMonthlySales(sc.CustomerID) as CustomerSales	
order by CustomerSales.TotalSales desc
go

--	4c. Find customers without sales this month, using the function
select	  sc.*
		, CustomerSales.TotalSales
from	sales.Customer as sc
		outer apply Sales.fnCustomerMonthlySales(sc.CustomerID) as CustomerSales	
order by CustomerSales.TotalSales asc
go

--	4d. Compare performance to 3c 